
from django import forms

from dynamic_rules import site

from dynamic_manipulation.dynamic_actions import BaseDynamicManipulation
from sample import models

@site.register
class FreeShippingPromotion(BaseDynamicManipulation):
    key = "free_shipping_promo"
    trigger_model_name = 'shopping_cart'

    fields = {
        'cart_total':forms.DecimalField(),
        'shipping_charge':forms.DecimalField(),
    }

    def do_manipulations(self, *args, **kwargs):
        if self.shopping_cart.total < self.rule_model.dynamic_fields['cart_total']:
            shipping_charge_amount = self.rule_model.dynamic_fields['shipping_charge']
            shipping_item = models.Item.objects.create(name="shipping charge", amount=shipping_charge_amount, cart=self.shopping_cart)
            self.log_manipulation(shipping_item)






from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models


class ManipulationLogManager(models.Manager):

    def get_by_rule(self, rule_model, trigger_model):
        content_type = ContentType.objects.get_for_model(trigger_model)
        return self.filter(
            trigger_content_type=content_type,
            trigger_model_id=trigger_model.pk,
            rule=rule_model,
        )

class ManipulationLog(models.Model):
    rule = models.ForeignKey('dynamic_rules.Rule', related_name='manipulation_logs')

    trigger_content_type = models.ForeignKey('contenttypes.ContentType', related_name="manipulation_triggers")
    trigger_model_id = models.PositiveIntegerField(db_index=True)
    trigger_model = generic.GenericForeignKey(fk_field='trigger_model_id', ct_field='trigger_content_type')

    side_effect_content_type = models.ForeignKey('contenttypes.ContentType', related_name="manipulation_side_effects")
    side_effect_model_id = models.PositiveIntegerField(db_index=True)
    side_effect_model = generic.GenericForeignKey(fk_field='side_effect_model_id', ct_field='side_effect_content_type')

    objects = ManipulationLogManager()




from decimal import Decimal
from hamcrest import assert_that, equal_to
from lettuce import step, world

from sample import models

@step(u'Given a shopping cart named "(.*)"')
def given_a_shopping_cart_named(step, cart_name):
    world.cart = models.Cart.objects.create(name=cart_name)

@step(u'(?:And|When) I add an item with a name of "(.*)" and price of "(.*)" dollars')
def given_a_shopping_cart_with_a_total_of_amount(step, item_name, amount):
    models.Item.objects.create(name=item_name, amount=amount, cart=world.cart)

@step(u'And I view my shopping cart')
def view_shopping_cart(step):
    world.cart.view_cart()

@step(u'Then I expect the cart total to be "(.*)" dollars')
def then_i_expect_a_discount_to_be_applied_to_the_shopping_cart(step, amount):
    assert_that(world.cart.total, equal_to(Decimal(amount)))

@step(u'Given I have a shopping cart with a total of "(.*)" dollars')
def given_i_have_a_shopping_cart_with_a_total(step, cart_total):
    world.cart = models.Cart.objects.create(name="new cart")
    models.Item.objects.create(name='new item', amount=cart_total, cart=world.cart)

@step(u'And I have a "(.*)" dollar shipping charge')
def and_i_have_a_shipping_charge(step, shipping_charge):
    world.cart.view_cart()



from django.utils import unittest
from django.contrib.contenttypes.models import ContentType
import mock

from dynamic_manipulation import models


class ModelTests(unittest.TestCase):

    def setUp(self):
        self.rule_model = mock.Mock()
        self.trigger_model = mock.Mock()

    @mock.patch.object(ContentType.objects, 'get_for_model')
    def test_get_content_type_for_model_in_get_by_validation_object(self, get_for_model):
        manager = mock.Mock(spec_set=models.ManipulationLogManager)
        models.ManipulationLogManager.get_by_rule(manager, self.rule_model, self.trigger_model)
        get_for_model.assert_called_once_with(self.trigger_model)

    @mock.patch.object(ContentType.objects, 'get_for_model')
    def test_get_by_validation_object_returns_rules_for_related_object(self, get_for_model):
        manager = mock.Mock(spec_set=models.ManipulationLogManager)
        logs = models.ManipulationLogManager.get_by_rule(manager, self.rule_model, self.trigger_model)

        manager.filter.assert_called_once_with(
            trigger_content_type=get_for_model.return_value,
            trigger_model_id=self.trigger_model.pk,
            rule=self.rule_model,
        )
        self.assertEqual(manager.filter.return_value, logs)


from lettuce import before

from django.db import connection
from django.core.management import call_command
from django.contrib.sites import models as site_models

from dynamic_rules import models as rule_models
from sample import dynamic_actions

@before.all
def setup_test_database():
    connection.creation.create_test_db(verbosity=1, autoclobber=True)

@before.each_scenario
def clean_db(scenario):
    call_command('flush', interactive=False)

    current_site = site_models.Site.objects.get_current()
    rule_models.Rule.objects.create(group_object=current_site,
                                    name="Free Shipping on orders over $10",
                                    key=dynamic_actions.FreeShippingPromotion.key,
                                    dynamic_fields=dict(cart_total=10, shipping_charge=2))



from django.db import models

from dynamic_rules import models as rule_models
from django.contrib.sites import models as site_models

class Cart(models.Model):
    name = models.CharField(max_length=30)

    def view_cart(self):
        current_site = site_models.Site.objects.get_current()
        rules = rule_models.Rule.objects.get_by_group_object(current_site)

        for rule in rules:
            rule.run_action(self)

    @property
    def total(self):
        return sum(i.amount for i in self.items.all())

class Item(models.Model):
    cart = models.ForeignKey(Cart, related_name="items")
    name = models.CharField(max_length=30)
    amount = models.DecimalField(decimal_places=2, max_digits=10)
    



from dynamic_rules.dynamic_actions import BaseDynamicAction

from dynamic_manipulation import models
from dynamic_manipulation.models import ManipulationLog

__all__ = ('BaseDynamicManipulation',)

class BaseDynamicManipulation(BaseDynamicAction):

    def clear_existing(self):
        """
        Both Side Effect Models and Manipulation Logs may have been
        created for a particular rule and trigger. This method
        deletes all of them.
        """
        manipulation_logs = models.ManipulationLog.objects.get_by_rule(self.rule_model, self.trigger_model)
        for log in manipulation_logs:
            log.side_effect_model.delete()
        manipulation_logs.delete()

    def run(self, *args, **kwargs):
        self.clear_existing()
        self.do_manipulations(*args, **kwargs)

    def do_manipulations(self, *args, **kwargs):
        """
        Implement this method on your custom rule classes.
        """
        pass

    def log_manipulation(self, side_effect_model):
        ManipulationLog.objects.create(
            rule=self.rule_model,
            trigger_model=self.trigger_model,
            side_effect_model=side_effect_model,
        )



from setuptools import setup, find_packages

REQUIREMENTS = [
    'django',
    'django-dynamic-rules',
]
TEST_REQUIREMENTS = [
    'mock',
    'lettuce',
    'pyhamcrest',
]

setup(
    name="django-dynamic-manipulation",
    version='0.0.1',
    author="Matthew J Morrison & Aaron Madison",
    author_email="mattjmorrison@mattjmorrison.com",
    description="Record rule based dynamic manipulations.",
    long_description=open('README.txt', 'r').read(),
    url="https://github.com/imtapps/django-dynamic-manipulation",
    packages=find_packages(exclude=["example"]),
    install_requires=REQUIREMENTS,
    tests_require=TEST_REQUIREMENTS,
    test_suite='runtests.runtests',
    zip_safe=False,
    classifiers = [
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
    ],
)




import mock
from django.utils import unittest

from dynamic_manipulation.dynamic_actions import BaseDynamicManipulation
from dynamic_manipulation import models

__all__ = (
    'BaseDynamicManipulationTests',
)

class BaseDynamicManipulationTests(unittest.TestCase):

    def setUp(self):
        self.rule_model = mock.Mock()
        self.trigger_model = mock.Mock()
        self.manipulation = BaseDynamicManipulation(self.rule_model, self.trigger_model)

    @mock.patch.object(BaseDynamicManipulation, 'do_manipulations', mock.Mock())
    @mock.patch.object(BaseDynamicManipulation, 'clear_existing')
    def test_run_calls_clear_existing(self, clear_existing):
        self.manipulation.run()
        clear_existing.assert_called_once_with()

    @mock.patch.object(BaseDynamicManipulation, 'clear_existing', mock.Mock())
    @mock.patch.object(BaseDynamicManipulation, 'do_manipulations')
    def test_run_calls_do_manipulations(self, do_manipulations):
        args = ["arg"]
        kwargs = {'kwarg': 'something'}

        self.manipulation.run(*args, **kwargs)
        do_manipulations.assert_called_once_with(*args, **kwargs)

    @mock.patch('dynamic_manipulation.models.ManipulationLog.objects.get_by_rule')
    def test_clear_existing_gets_manipulation_logs_by_rule_model(self, get_by_rule):
        get_by_rule.return_value = mock.MagicMock()

        self.manipulation.clear_existing()
        get_by_rule.assert_called_once_with(self.rule_model, self.trigger_model)

    @mock.patch('dynamic_manipulation.models.ManipulationLog.objects.get_by_rule')
    def test_clear_existing_deletes_side_effects_related_to_manipulation_log(self, get_by_rule):
        log_one = mock.Mock(spec_set=models.ManipulationLog())
        log_two = mock.Mock(spec_set=models.ManipulationLog())
        logs = mock.MagicMock()
        logs.__iter__.return_value = iter([log_one, log_two])

        get_by_rule.return_value = logs

        self.manipulation.clear_existing()

        log_one.side_effect_model.delete.assert_called_once_with()
        log_two.side_effect_model.delete.assert_called_once_with()
        logs.delete.assert_called_once_with()

    @mock.patch.object(models.ManipulationLog.objects, 'create')
    def test_log_manipulation_logs_manipulation_with_rule_trigger_model_and_side_effect_model(self, create_log):
        side_effect_model = mock.Mock()

        self.manipulation.log_manipulation(side_effect_model)
        create_log.assert_called_once_with(
            rule=self.rule_model,
            trigger_model=self.trigger_model,
            side_effect_model=side_effect_model,)



        
        
